# Copyright 2017 Jose Nelson Amaral
# Copyright 2017 Erick Ochoa Lopez
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from setting_values import *


__all__ = ['TRAIN']

TRAIN = {}

Global_options = {
	'Output directory': GO_Output_directory(default='.'),
	'Output format': GO_Output_format(default='vtk'),
	'Number of experiments': GO_Number_of_experiments(default=4),
	'Data': GO_Data(default='single target'),
	'Coarse mesh': GO_Coarse_mesh(default='8x8x4'),
}

Experiment_description = {
	'Forcing mode': ED_Forcing_mode(default='gaussian'),
}

Measurement_representation = {
	'Measurement type': MR_Measurement_type(default='synthetic'),
}
Experiment_description['Measurement representation'] = Measurement_representation

Synthetic_data = {
	'Finite element': SD_Finite_element(default='linear'),
	'Noise level': SD_Noise_level(default=0.0),
	'Noise type': SD_Noise_type(default='multiplicative'),
	'Output projected solution': SD_Output_projected_solution(default=False),
	'Unknown source amplitude': SD_Unknown_source_amplitude(default=1.0),
	'Unknown source phase': SD_Unknown_source_phase(default=0.0),
}
Measurement_representation['Synthetic data'] = Synthetic_data

Discretization = {}

Distributed_parameter_field = {
	'Finite element': DPF_Finite_element(default='discontinous constant'),
	'Regularization functional': DPF_Regularization_functional(default='L2 norm'),
	'Regularization parameter': DPF_Regularization_parameter(default=1e-7),
	'Initial refinement': DPF_Initial_refinement(default=0),
	'Refinements on measurement boundary': DPF_Refinements_on_measurement_boundary(default='g'),
	'Refinement fraction': DPF_Refinement_fraction(default=0.4),
	'Coarsening fraction': DPF_Coarsening_fraction(default=0.05),
	'Lower bound': DPF_Lower_bound(default=0.0001),
	'Upper bound': DPF_Upper_bound(default=0.002),
}

Regularization_strategy = {
	'Strategy': RS_Strategy(default='reduce with misfit'),
	'Minimal misfit ratio': RS_Minimal_misfit_ratio(default=3),
}
Distributed_parameter_field['Regularization strategy'] = Regularization_strategy
Discretization['Distributed parameter field'] = Distributed_parameter_field

Tomography_slave = {
	'Finite element': TS_Finite_element(default='linear'),
	'Misfit functional': TS_Misfit_functional(default='L2 boundary norm'),
	'Renumber dofs by Cuthill-McKee': TS_Renumber_dofs_by_Cuthill_McKee(default=False),
	'Initial refinement': TS_Initial_refinement(default=0),
	'Refinements on measurement boundary': TS_Refinements_on_measurement_boundary(default='gmg'),
	'Refinement criterion': TS_Refinement_criterion(default='grad grad u'),
	'Refinement fraction': TS_Refinement_fraction(default=0.35),
	'Coarsening fraction': TS_Coarsening_fraction(default=0.05),
	'Output grids': TS_Output_grids(default=False),
	'Evaulations': TS_Evaluations(default=''),
	'Output magnitude and phase': TS_Output_magnitude_and_phase(default=True),
}

Discretization['Tomography slave'] = Tomography_slave

Newton_method = {
	'Max inner CG iterations': NM_Max_inner_CG_iterations(default=3000),
	'Reduction goal': NM_Reduction_goal(default=1e-3),
	'Output step interval': NM_Output_step_interval(default=1),
	'Maximal number of iterations': NM_Maximal_number_of_iterations(default=2),
	'Target measurement misfit': NM_Target_measurement_misfit(default=0),
	'Solver type': NM_Solver_type(default='cg'),
	'Write hessian': NM_Write_hessian(default=False)
}

Mesh_refinement_details = {
	'Min initial steps': MRD_Min_initial_steps(default=2),
	'Max initial steps': MRD_Max_initial_steps(default=4),
	'Initial reduction': MRD_Initial_reduction(default=1e-10),
	'Reduction per mesh': MRD_Reduction_per_mesh(default=1e-4),
}

Newton_method['Mesh refinement details'] = Mesh_refinement_details

TRAIN['Global options'] = Global_options
TRAIN['Experiment description'] = Experiment_description
TRAIN['Discretization'] = Discretization
TRAIN['Newton method'] = Newton_method
