# Copyright 2017 Jose Nelson Amaral
# Copyright 2017 Erick Ochoa Lopez
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import random
from inrand import *


@decorators(retStr, forceRetVal)
def GO_Output_directory(*args, **kwargs):
	return kwargs.get('default')

@decorators(retStr, forceRetVal)
def GO_Output_format(*args, **kwargs):
	return kwargs.get('default')

@decorators(retStr, retAbsVal, forceRetVal)
def GO_Number_of_experiments(*args, **kwargs):	
	return randint(kwargs.get('default'))

@decorators(retStr, forceRetVal)
def GO_Data(*args, **kwargs):
	return kwargs.get('default', 'single target')

@decorators(retStr, forceRetVal)
def GO_Coarse_mesh(*args, **kwargs):
	return kwargs.get('default', '8x8x4')

@decorators(retStr, forceRetVal)
def ED_Forcing_mode(*args, **kwargs):
	return kwargs.get('default', 'gaussian')

@decorators(retStr, forceRetVal)
def MR_Measurement_type(*args, **kwargs):
	return kwargs.get('default', 'synthetic')

@decorators(retStr, forceRetVal)
def SD_Finite_element(*args, **kwargs):
	return random.choice(['linear', 'quadratic', 'cubic'])

@decorators(retStr, retAbsVal, forceRetVal)
def SD_Noise_level(*args, **kwargs):
	return random.uniform(0, 1)

@decorators(retStr, forceRetVal)
def SD_Noise_type(*args, **kwargs):
	return random.choice(['additive', 'multiplicative'])

@decorators(toLow, retStr, forceRetVal)
def SD_Output_projected_solution(*args, **kwargs):
	return random.choice([True, False])

@decorators(retStr, retAbsVal, forceRetVal)
def SD_Unknown_source_amplitude(*args, **kwargs):
	return uniform(kwargs.get('default'))

@decorators(retStr, retAbsVal, forceRetVal)
def SD_Unknown_source_phase(*args, **kwargs):
	return random.uniform(0.0, 360.0)

@decorators(retStr)
def DPF_Finite_element(*args, **kwargs):
	return kwargs.get('default')

@decorators(retStr, forceRetVal)
def DPF_Regularization_functional(*args, **kwargs):
	return random.choice(['L2 norm', 'H1 seminorm'])

@decorators(retStr, forceRetVal)
def DPF_Regularization_parameter(*args, **kwargs):
	return random.uniform(0.0, 1e-7)

@decorators(retStr, retAbsVal, forceRetVal)
def DPF_Initial_refinement(*args, **kwargs):
	return randint(kwargs.get('default', 0))

@decorators(retStr, forceRetVal)
def DPF_Refinements_on_measurement_boundary(*args, **kwargs):
	return random.choice(['g', 'm'])

@decorators(retStr, forceRetVal)
def DPF_Refinement_fraction(*args, **kwargs):
	return random.uniform(0.0, 1.0)

@decorators(retStr, forceRetVal)
def DPF_Coarsening_fraction(*args, **kwargs):
	return random.uniform(0.0, 0.1)

@decorators(retStr, forceRetVal)
def DPF_Lower_bound(*args, **kwargs):
	return random.uniform(0.0, 0.0002)

@decorators(retStr, forceRetVal)
def DPF_Upper_bound(*args, **kwargs):
	return random.uniform(0.1, 2)

@decorators(forceRetVal)
def RS_Strategy(*args, **kwargs):
	return random.choice(['reduce with misfit', 'fixed'])

@decorators(retStr, retAbsVal, forceRetVal)
def RS_Minimal_misfit_ratio(*args, **kwargs):
	return uniform(kwargs.get('default'))

@decorators(forceRetVal)
def TS_Finite_element(*args, **kwargs):
	return random.choice(['linear', 'quadratic'])

@decorators(forceRetVal)
def TS_Misfit_functional(*args, **kwargs):
	return random.choice(['L2 norm',
			'L2 boundary norm',
			'L2 weighted boundary norm',
			'L2 weighted boundary norm (real scaling)',
			'L2 weighted boundary norm (complex scaling)',
			'ratio over ratio minus one',
			'ratio minus ratio'])

@decorators(toLow, retStr, forceRetVal)
def TS_Renumber_dofs_by_Cuthill_McKee(*args, **kwargs):
	return random.choice([True, False])

@decorators(retStr, retAbsVal, forceRetVal)
def TS_Initial_refinement(*args, **kwargs):
	return randint(kwargs.get('default'))

@decorators(forceRetVal)
def TS_Refinements_on_measurement_boundary(*args, **kwargs):
	return kwargs.get('default', 'gmg')

@decorators(forceRetVal)
def TS_Refinement_criterion(*args, **kwargs):
	return random.choice(['lambda',
			'grad u',
			'grad lambda',
			'grad grad u',
			'grad grad lambda',
			'weighted grad grad',
			'global'])

@decorators(retStr, forceRetVal)
def TS_Refinement_fraction(*args, **kwargs):
	return random.uniform(0.001, 1)

@decorators(retStr, forceRetVal)
def TS_Coarsening_fraction(*args, **kwargs):
	return random.uniform(0.001, 1)

@decorators(toLow, retStr, forceRetVal)
def TS_Output_grids(*args, **kwargs):
	return random.choice([True, False])

@decorators(forceRetVal)
def TS_Evaluations(*args, **kwargs):
	return kwargs.get('default', '')

@decorators(toLow, retStr, forceRetVal)
def TS_Output_magnitude_and_phase(*args, **kwargs):
	return random.choice([True, False])

@decorators(retStr, forceRetVal)
def MRD_Min_initial_steps(*args, **kwargs):
	return randint(kwargs.get('default'))

@decorators(retStr, forceRetVal)
def MRD_Max_initial_steps(*args, **kwargs):
	return randint(kwargs.get('default'))

@decorators(retStr, retSciNot, forceRetVal)
def MRD_Initial_reduction(*args, **kwargs):
	return random.uniform(0, 2e-10)

@decorators(retStr, retSciNot, forceRetVal)
def MRD_Reduction_per_mesh(*args, **kwargs):
	return random.uniform(0, 2e-4)

@decorators(retStr, retAbsVal, forceRetVal)
def NM_Max_inner_CG_iterations(*args, **kwargs):
	return randint(kwargs.get('default'))

@decorators(retStr, retSciNot, retAbsVal, forceRetVal)
def NM_Reduction_goal(*args, **kwargs):
	return random.uniform(0, 2e-3)

def NM_Output_step_interval(*args, **kwargs):
	return random.randint(1, 5)

@decorators(retStr, retAbsVal, forceRetVal)
def NM_Maximal_number_of_iterations(*args, **kwargs):
	return randint(kwargs.get('default'))

@decorators(retStr, retAbsVal, forceRetVal)
def NM_Target_measurement_misfit(*args, **kwargs):
	return uniform(kwargs.get('default'))

@decorators(forceRetVal)
def NM_Solver_type(*args, **kwargs):
	return random.choice(['cg', 'direct'])

@decorators(toLow, retStr, forceRetVal)
def NM_Write_hessian(*args, **kwargs):
	return random.choice([True, False])
