#!/usr/bin/env python

# Copyright 2017 Jose Nelson Amaral
# Copyright 2017 Erick Ochoa Lopez
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
import unittest
from ingen import inrand

@inrand.retStr
def foo(*args, **kwargs):
	return True

@inrand.forceRetVal
def bar(*args, **kwargs):
	return False

@inrand.toLow
def baz(*args, **kwargs):
	return "True"

@inrand.retSciNot
def qux(*args, **kwargs):
	return 1e-7

@inrand.decorators(inrand.retStr, inrand.retSciNot, inrand.retAbsVal, inrand.forceRetVal)
def all(*args, **kwargs):
	return 1e-7

class InRandTest(unittest.TestCase):

	def testRetStr(self):
		expected = "True"
		observed = foo()
		self.assertEqual(expected, observed)

	def testForceRetVal(self):
		expected = True
		observed = bar(retVal=True)
		self.assertEqual(expected, observed)

	def testToLow(self):
		expected = "true"
		observed = baz()
		self.assertEqual(expected, observed)

	def testRetScieNot(self):
		expected = "1.000000e-07"
		observed = qux()
		self.assertEqual(expected, observed)

	def testDecorators(self):
		expected = "1.000000e-02"
		observed = all(retVal=-1.000000e-2)
		self.assertEqual(expected, observed)
